
#*********************************************************************
#
# Copyright 2016 Broadcom.
#
#  Licensed under the Apache License, Version 2.0 (the "License")
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#***********************************************************************

class Gateway():

    '''
    Class to abstract a VTEP gateway. Used directly by configuration
    scripts.
    '''

    def __init__(self):
        self._ip = None
        self._vlans = []
        self._devices = []

    def set_ip(self, ip):
        self._ip = ip

    def get_ip(self):
        return self._ip

    def add_vlan(self, number, name):
        vlan = {"name": name, "number": number, "ports": set()}
        self._vlans.append(vlan)

    def add_port_to_vlan(self, vlan, port):
        for y in self._vlans:
            if y['number'] == vlan:
                y["ports"].add(port)
                break

    def is_port_in_vlan(self, port, vlan):
        ret = False
        for x in self._vlans:
            if x['number'] == number and port in x["ports"]:
                ret = True
                break
        return ret

    def get_port_vlans(self, port):
        ret = []
        for x in self._vlans:
            if port in x["ports"]:
                ret.append(x["number"])
        return ret

    def get_vlan(self, vlan):
        ret = None
        for x in self._vlans:
            if x["number"] == vlan:
                ret = x
                break
        return ret

    def add_device(self, device):
        self._devices.append(device)

    def get_devices(self):
        return self._devices

    def get_device_by_name(self, name):
        ret = None
        for x in self._devices:
            if x.get_name() == name:
                ret = x
        return ret

class Device():
    '''
    Class to abstract a device plugged into a VTEP gateway. Used directly 
    by configuration scripts.
    '''

    def __init__(self):
        self._name = None
        self._description = None
        self._port = None
        self._mac = None  
        self._gateway = None

    def set_mac(self, mac):
        self._mac = mac

    def get_mac(self):
        return self._mac

    def set_name(self, name):
        self._name = name

    def get_name(self):
        return self._name

    def set_description(self, description):
        self._description = description

    def get_description(self):
        return self._description

    def set_port(self, port):
        self._port = port
        
    def get_port(self):
        return self._port

    def set_gateway(self, gateway):
        self._gateway = gateway
        
    def get_gateway(self):
        return self._gateway

